@@ -1,6 +1,6 @@
 /*
 * cdpr - Cisco Discovery Protocol Reporter
-* Copyright (c) 2002-2006 MonkeyMental.com
+* Copyright (c) 2002-2010 MonkeyMental.com
 *
 * This program will show you which Cisco device your machine is
 * connected to based on CDP packets received.
@@ -58,6 +58,8 @@
 * 2.1.0	LO	03-11-08	Release - Got timeout code working on WIN32 port
 * 2.2.0	LO	04-04-14	Added the ability specify the port to send the cdpr updates to.
 * 2.2.1 LO	06-07-28	Fixed segfault when not providing port to a command line selected server (-s)
+* 2.3.0 LO	09-??-??	Decode IPv6 addresses
+* 2.4.0 LO	10-02-15	Add ioctl to OSX to disable buffers, add ability to read from file (-r)
 */
 
 /*#include "pcap.h" */
@@ -65,6 +67,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <sys/socket.h>
+#include <arpa/inet.h>
+/* apple stuff */
+#if defined(__APPLE_CC__) || defined(__APPLE__)
+#include <sys/ioctl.h>
+#include <net/bpf.h>
+#endif
+/* end apple stuff */
 #ifdef WIN32
 #include "xgetopt.h"
 #include "time.h"
@@ -83,7 +93,7 @@ dump_ip (const u_char *ip)
 	char switch_ip[100];
 	if(cdprs>=1)
 	{
-		sprintf (switch_ip, "switch_ip=%d.%d.%d.%d",
+		sprintf (switch_ip, "switch_ip=%d.%d.%d.%d&",
 			(int) ip[0], (int) ip[1], (int) ip[2], (int) ip[3]);
 		cdprs_action(CDPRS_DATA, switch_ip, 0);
 	}
@@ -91,6 +101,24 @@ dump_ip (const u_char *ip)
 		(int) ip[0], (int) ip[1], (int) ip[2], (int) ip[3]);
 }
 
+void
+dump_ipv6 (const u_char *ip)
+{
+    char ipv6_buf[INET6_ADDRSTRLEN];
+    struct in6_addr addr;
+    register const char *p_ipv6;
+    char switch_ip6[600];
+
+    memcpy(&addr, ip, sizeof(addr));
+    p_ipv6 = inet_ntop(AF_INET6, &addr, ipv6_buf, sizeof(ipv6_buf));
+    printf("%s", ipv6_buf);
+    if(cdprs>=1)
+    {
+        sprintf(switch_ip6, "switch_ip6=%s&", ipv6_buf);
+        cdprs_action(CDPRS_DATA, switch_ip6, 0);
+    }
+}
+
 void
 dump_hex (const u_char *p, int len)
 {
@@ -184,6 +212,9 @@ print_cdp_address (u_char *v, int verbose)
 		}
 		if (protocol_len == 1 && *protocol_val == 0xCC && address_len == 4)
 			dump_ip (address_val);
+//        else if (protocol_len == 8 && *protocol_val == "0xAAAA0300000086DD" && address_len == 16)
+        else if (protocol_len == 8 && address_len == 16)
+			dump_ipv6 (address_val);
 		else
 			dump_hex (address_val, address_len);
 		printf ("\n");
@@ -272,8 +303,8 @@ print_cdp_packet (const u_char *p, unsigned int plen, int verbose)
 				char port[1024];
 				char *portval;
 				int portlen;
-				portval = urlencode(v, strlen(v), &portlen);
-				sprintf(port, "&port=%.*s", portlen, portval);
+				portval = urlencode((char *)v, strlen((char *)v), &portlen);
+				sprintf(port, "port=%.*s", portlen, portval);
 				free(portval);
 				cdprs_action(CDPRS_DATA, port, verbose);
 			}
@@ -449,6 +480,7 @@ usage(void)
 	puts("h: Print this usage");
 	puts("t: time in seconds to abort waiting for a packet (should be > 60)");
 	puts("v[vv]: Set verbose mode");
+	puts("r: read from file instead of network device");
 	puts("\n** Options dealing with server updates: **");
 	puts(" u: Send cdpr information to a cdpr server\n    requires config file as arg");
 	puts(" l: Location/description of this port for use with -u or -s and -p");
@@ -469,6 +501,8 @@ main(int argc, char *argv[])
 	char *name = NULL;
 	char *svr = NULL;
 	char *path = NULL;
+	const char *fname = NULL;
+	int rff = 0;
 	char errbuf[PCAP_ERRBUF_SIZE];
 	struct bpf_program filter;
 	/*
@@ -484,8 +518,8 @@ main(int argc, char *argv[])
 	bpf_u_int32 mask;
 	bpf_u_int32 net;
 	struct pcap_pkthdr header;
-	const u_char *packet;
-	char version[] = "2.2.1";
+	const u_char *packet = NULL;
+	char version[] = "2.4";
 
 	int c;
 	int verbose=0;
@@ -503,10 +537,10 @@ main(int argc, char *argv[])
 
 	/* Print out header */
 	printf("cdpr - Cisco Discovery Protocol Reporter\nVersion %s\n", version);
-	printf("Copyright (c) 2002-2006 - MonkeyMental.com\n\n");
+	printf("Copyright (c) 2002-2010 - MonkeyMental.com\n\n");
 
 	/* Check command-line options */
-	while((c = getopt(argc, argv, "d:t:vhu:l:n:s:p:")) !=EOF)
+	while((c = getopt(argc, argv, "d:t:vhu:l:n:s:p:r:")) !=EOF)
 		switch(c)
 		{
 			case 'd':
@@ -562,6 +596,10 @@ main(int argc, char *argv[])
 					cdprs_cmdline++;
 				}
 				break;
+			case 'r':
+				fname = optarg;
+				rff = 1;
+				break;
 			case 'h':
 			case '?':
 				usage();
@@ -585,7 +623,7 @@ main(int argc, char *argv[])
 	}
 
 	/* Get a pcap capable device */
-	if(dev == NULL)
+	if((dev == NULL) && (rff == 0))
 	{
 		int i = 0;
 		int inum;
@@ -635,18 +673,35 @@ main(int argc, char *argv[])
 		dev = d->name;
 	}
 
-	printf("Using Device: %s\n", dev);
 
 	/* Get the network number and netmask */
 	pcap_lookupnet(dev, &net, &mask, errbuf);
 
 	/* Open the pcap device */
-	if((handle = pcap_open_live(dev, BUFSIZ, 1, 0, errbuf)) == NULL)
+	if(rff == 0)
 	{
-		printf("Error opening device (%s)\n", errbuf);
-		exit(1);
+		printf("Using Device: %s\n", dev);
+		if((handle = pcap_open_live(dev, BUFSIZ, 1, 1000, errbuf)) == NULL)
+		{
+			printf("Error opening device (%s)\n", errbuf);
+			exit(1);
+		}
+#if defined(__APPLE_CC__) || defined(__APPLE__)
+	      int v = 1;
+	      ioctl(pcap_fileno(handle), BIOCIMMEDIATE, &v);
+#endif
+
 	}
-	else if (strlen(errbuf))
+	else
+	{
+		printf("Using File %s\n", fname);
+		if((handle = pcap_open_offline(fname, errbuf)) == NULL)
+		{
+			printf("Error opening file (%s)\n", fname);
+			exit(1);
+		}
+	}
+	if (strlen(errbuf))
 	{
 		printf("Warning opening device (%s)\n", errbuf);
 	}
@@ -678,7 +733,7 @@ main(int argc, char *argv[])
 #else
 		usleep(10000);
 #endif
-	} while ((!packet) && ( timeout=((start_time+seconds) > (unsigned int)time(NULL))) );
+	} while ((packet == NULL) && ( timeout=((start_time+seconds) > (unsigned int)time(NULL))) );
 
 	/*
 	** timeout expired. clean up and exit
