@@ -2,7 +2,7 @@
 /*
 ** cdprs.php - Sample PHP script to collect cdpr data to a file
 **
-** Copyright (c) 2002-2006 MonkeyMental.com
+** Copyright (c) 2002-2008 MonkeyMental.com
 **
 ** This program will show you which Cisco device your machine is
 ** connected to based on CDP packets received.
@@ -24,6 +24,7 @@
 	header("Content-type: plain/text");
 
 	$switch = $_GET["switch_ip"];
+	$switch6 = $_GET["switch_ip6"];
 	$port = $_GET["port"];
 	$host = $_GET["host"];
 	$loc = $_GET["loc"];
@@ -34,8 +35,8 @@
 		exit();
 	}
 
-	$cdprs = sprintf("Switch: %s Port: $port Host: $host Location: $loc\n", 
-					  $switch, $port, $host, $loc);
+	$cdprs = sprintf("Switch: %s/%s Port: $port Host: $host Location: $loc\n", 
+					  $switch, $switch6, $port, $host, $loc);
 
 	if(!fwrite($fh, $cdprs))
 	{
